@@ -78,22 +78,21 @@ public:
  */
 class UserTime {
 private:
-    unsigned long xtime;
+    unsigned xtime;
     bool valid;
-    unsigned long since;
-    enum : unsigned long {
-        XTimeMask = 0xFFFFFFFFUL,       // milliseconds
-        XTimeRange = 0x7FFFFFFFUL,      // milliseconds
-        SInterval = 0x3FFFFFFFUL / 1000,     // seconds
+    unsigned since;
+    enum : unsigned {
+        XTimeMask = 0xFFFFFFFFU,       // milliseconds
+        XTimeRange = 0x7FFFFFFFU,      // milliseconds
+        SInterval = 0x3FFFFFFFU / 1000,     // seconds
     };
 public:
     UserTime() : xtime(0), valid(false), since(0) { }
-    explicit UserTime(unsigned long xtime, bool valid = true) :
+    explicit UserTime(unsigned xtime, bool valid = true) :
         xtime(xtime & XTimeMask), valid(valid), since(seconds()) { }
-    unsigned long time() const { return xtime; }
+    unsigned time() const { return xtime; }
     bool good() const { return valid; }
-    long when() const { return since; }
-    bool update(unsigned long xtime, bool valid = true) {
+    bool update(unsigned xtime, bool valid = true) {
         UserTime u(xtime, valid);
         return *this < u || xtime == 0 ? (*this = u, true) : false;
     }
@@ -117,9 +116,11 @@ public:
     virtual mstring getTitle() const = 0;
     virtual mstring getIconTitle() const = 0;
     virtual void activateWindow(bool raise, bool curWork) = 0;
+    virtual bool isFullscreen() const = 0;
     virtual bool isMinimized() const = 0;
     virtual bool focused() const = 0;
     virtual bool visibleNow() const = 0;
+    virtual bool canMinimize() const = 0;
     virtual bool canRaise(bool ignoreTaskBar = false) const = 0;
     virtual void wmClose() = 0;
     virtual void wmRaise() = 0;
@@ -164,6 +165,7 @@ public:
     virtual void handleUnmap(const XUnmapEvent &unmap);
     virtual void handleDestroyWindow(const XDestroyWindowEvent &destroyWindow);
     virtual void handleClientMessage(const XClientMessageEvent &message);
+    virtual void handleDamageNotify(const XDamageNotifyEvent& damage);
 #ifdef CONFIG_SHAPE
     virtual void handleShapeNotify(const XShapeEvent &shape);
 #endif
@@ -262,6 +264,8 @@ public:
     void refreshIcon();
     void obtainIcon();
     ref<YIcon> getIcon();
+    bool haveMetaIcon() const { return fMetaIcon; }
+    ref<YImage> getMetaIcon();
 
     void setWorkspaceHint(int workspace);
 
@@ -284,9 +288,9 @@ public:
     bool getNetWMPid(long *pid);
     bool getNetWMStrut(int *left, int *right, int *top, int *bottom);
     bool getNetWMStrutPartial(int *left, int *right, int *top, int *bottom);
-    bool getNetStartupId(unsigned long &time);
-    bool getNetWMUserTime(Window window, unsigned long &time);
-    bool getNetWMUserTimeWindow(Window &window);
+    bool getNetStartupId(unsigned& time);
+    bool getUserTime(Window window, unsigned& time);
+    bool getUserTimeWindow();
     bool getNetWMWindowOpacity(long &opacity);
     bool getNetWMWindowType(WindowType *window_type);
     void setNetWMFullscreenMonitors(int top, int bottom, int left, int right);
@@ -328,6 +332,9 @@ public:
     bool activateOnMap();
     YAction action();
 
+    bool getUserTime();
+    const UserTime& userTime() const { return fUserTime; }
+
 private:
     YFrameWindow *fFrame;
     YClientItem* fClientItem;
@@ -345,6 +352,7 @@ private:
     bool fTimedOut;
     bool fFixedTitle;
     bool fIconize;
+    bool fMetaIcon;
     bool fPinging;
     long fPingTime;
     long fPid;
@@ -363,6 +371,9 @@ private:
     lazy<WindowOption> fWindowOption;
     void loadWindowOptions(WindowOptions* list, bool remove);
 
+    UserTime fUserTime;
+    Window fUserTimeWindow;
+
     struct transience {
         Window trans, owner;
         transience(Window t, Window o) : trans(t), owner(o) { }
@@ -406,6 +417,7 @@ private:
         bool win_icons : 1;
         bool xembed_info : 1;
     } prop;
+
 private: // not-used
     YFrameClient(const YFrameClient &);
     YFrameClient &operator=(const YFrameClient &);
