@@ -18,6 +18,7 @@
 #include "wmtaskbar.h"
 
 #include <string.h>
+#include <ctype.h>
 
 #include "intl.h"
 
@@ -27,8 +28,8 @@ YColorName YButton::normalButtonFg(&clrNormalButtonText);
 YColorName YButton::activeButtonBg(&clrActiveButton);
 YColorName YButton::activeButtonFg(&clrActiveButtonText);
 
-ref<YFont> YButton::normalButtonFont;
-ref<YFont> YButton::activeButtonFont;
+YFont YButton::normalButtonFont;
+YFont YButton::activeButtonFont;
 int YButton::buttonObjectCount;
 
 YButton::YButton(YWindow *parent, YAction action, YMenu *popup) :
@@ -58,7 +59,7 @@ YButton::~YButton() {
     }
     popdown();
     if (fPopup && fPopup->isShared() == false) {
-        delete fPopup;
+        delete fPopup; fPopup = nullptr;
     }
     if (--buttonObjectCount == 0) {
         normalButtonFont = null;
@@ -68,8 +69,6 @@ YButton::~YButton() {
 
 void YButton::paint(Graphics &g, int const d, const YRect &r) {
     int x = r.x(), y = r.y(), w = r.width(), h = r.height();
-    YSurface surface(getSurface());
-    g.drawSurface(surface, x, y, w, h);
 
     if (fIcon != null) {
         fIcon->draw(g,
@@ -83,25 +82,26 @@ void YButton::paint(Graphics &g, int const d, const YRect &r) {
                     y + (h - fImage->height()) / 2);
     }
     else if (fText != null) {
-        ref<YFont> font(getFont());
+        YFont font(getFont());
+        if (font) {
+            int const w(font->textWidth(fText));
+            int const p((width() + 1 - w) / 2);
+            int yp((height() - font->height()) / 2
+                   + font->ascent() + d);
 
-        int const w(font->textWidth(fText));
-        int const p((width() + 1 - w) / 2);
-        int yp((height() - font->height()) / 2
-               + font->ascent() + d);
-
-        g.setFont(font);
-        g.setColor(getColor());
-        if (!fEnabled) {
-            g.setColor(YColor::white);
-            g.drawChars(fText, d + p + 1, yp + 1);
-            g.setColor(YColor::white->darker().darker());
-            g.drawChars(fText, d + p, yp);
-        } else {
-            g.drawChars(fText, d + p, yp);
+            g.setFont(font);
+            g.setColor(getColor());
+            if (!fEnabled) {
+                g.setColor(YColor::white);
+                g.drawChars(fText, d + p + 1, yp + 1);
+                g.setColor(YColor::white->darker().darker());
+                g.drawChars(fText, d + p, yp);
+            } else {
+                g.drawChars(fText, d + p, yp);
+            }
+            if (fHotCharPos >= 0)
+                g.drawCharUnderline(d + p, yp, fText, fHotCharPos);
         }
-        if (fHotCharPos >= 0)
-            g.drawCharUnderline(d + p, yp, fText, fHotCharPos);
     }
 }
 
@@ -111,6 +111,7 @@ void YButton::paint(Graphics &g, const YRect &/*r*/) {
 
     if (w > 1 && h > 1) {
         YSurface surface(getSurface());
+        g.drawSurface(surface, 0, 0, width(), height());
         g.setColor(surface.color);
 
         if (wmLook == lookMetal) {
@@ -128,7 +129,7 @@ void YButton::paint(Graphics &g, const YRect &/*r*/) {
 
         paint(g, d, YRect(x, y, w, h));
 
-        if (wmLook != lookFlat) {
+        if (wmLook != lookFlat || isFocused()) {
             paintFocus(g, YRect(x, y, w, h));
         }
         if (surface.gradient != null) {
@@ -145,7 +146,7 @@ void YButton::paintFocus(Graphics &g, const YRect &/*r*/) {
     if (isFocused()) {
         g.setPenStyle(true);
         g.setFunction(GXxor);
-        g.setColor(YColor::white);
+        g.setColorPixel(0x01FFFFFF);
         g.drawRect(dp, dp, width() - ds - 1, height() - ds - 1);
         g.setFunction(GXcopy);
         g.setPenStyle(false);
@@ -167,7 +168,7 @@ void YButton::paintFocus(Graphics &g, const YRect &/*r*/) {
             paint(g, 0, YRect(dp, dp, width() - ds, height() - ds));
         else
             paint(g, d, YRect(dp - 1, dp - 1, width() - ds + 1, height() - ds + 1));
-        g.setClipMask(None);
+        g.resetClip();
     }
 }
 
@@ -208,7 +209,7 @@ void YButton::setArmed(bool armed, bool mouseDown) {
 bool YButton::handleKey(const XKeyEvent &key) {
     KeySym k = keyCodeToKeySym(key.keycode);
     unsigned m = KEY_MODMASK(key.state);
-    int uk = (k < 256) ? ASCII::toUpper((char)k) : k;
+    int uk = (k < 256) ? toupper((unsigned char) k) : k;
 
     if (fEnabled) {
         if (key.type == KeyPress) {
@@ -289,7 +290,15 @@ void YButton::handleButton(const XButtonEvent &button) {
             setSelected(false);
         }
     }
+    bool unarm = fEnabled && fArmed
+              && button.type == ButtonRelease
+              && button.button == Button1
+              && getClickCount() == 1
+              && dragging();
     YWindow::handleButton(button);
+    if (unarm && !fPopupActive) {
+        setArmed(false, false);
+    }
 }
 
 void YButton::handleClick(const XButtonEvent &button, int count) {
@@ -326,30 +335,32 @@ void YButton::handleCrossing(const XCrossingEvent &crossing) {
     YWindow::handleCrossing(crossing);
 }
 
-ref<YFont> YButton::getActiveFont() {
+YFont YButton::getActiveFont() {
     if (activeButtonFont == null)
-        activeButtonFont = YFont::getFont(XFA(activeButtonFontName));
+        activeButtonFont = activeButtonFontName;
     return activeButtonFont;
 }
 
-ref<YFont> YButton::getNormalFont() {
+YFont YButton::getNormalFont() {
     if (normalButtonFont == null)
-        normalButtonFont = YFont::getFont(XFA(normalButtonFontName));
+        normalButtonFont = normalButtonFontName;
     return normalButtonFont;
 }
 
 YDimension YButton::getTextSize() {
+    YDimension dim(1, 1);
     if (fText != null) {
-        ref<YFont> font(getActiveFont());
-        return YDimension(font->textWidth(fText), font->height());
-    } else {
-        return YDimension(1, 1);
+        YFont font(getActiveFont());
+        if (font) {
+            dim = YDimension(font->textWidth(fText), font->height());
+        }
     }
+    return dim;
 }
 
 void YButton::updateSize() {
-    int w = 72;
-    int h = 18;
+    unsigned w = 72;
+    unsigned h = 18;
     if (fIcon != null) {
         w = h = fIconSize;
     }
@@ -362,24 +373,33 @@ void YButton::updateSize() {
         w = d.w;
         h = d.h;
     }
-    setSize(w + 3 + 2 - LOOK(lookMetal | lookFlat),
-            h + 3 + 2 - LOOK(lookMetal | lookFlat));
+    unsigned ww = w + 3 + 2 - LOOK(lookMetal | lookFlat);
+    unsigned hh = h + 3 + 2 - LOOK(lookMetal | lookFlat);
+    setSize(ww, hh);
 }
 
 void YButton::setIcon(ref<YIcon> icon, int iconSize) {
-    fIcon = icon;
-    fIconSize = iconSize;
-    fImage = null;
+    if (icon != fIcon || iconSize != fIconSize || fImage != null) {
+        fIcon = icon;
+        fIconSize = iconSize;
+        fImage = null;
 
-    updateSize();
+        updateSize();
+    }
+}
+
+ref<YIcon> YButton::getIcon() {
+    return fIcon;
 }
 
 void YButton::setImage(ref<YImage> image) {
-    fImage = image;
-    fIconSize = 0;
-    fIcon = null;
+    if (image != fImage || fIcon != null || fIconSize != 0) {
+        fImage = image;
+        fIconSize = 0;
+        fIcon = null;
 
-    updateSize();
+        updateSize();
+    }
 }
 
 void YButton::setText(const mstring &str, int hotChar) {
@@ -402,14 +422,16 @@ void YButton::setText(const mstring &str, int hotChar) {
 
         hotKey = (fHotCharPos >= 0) ? fText.charAt(fHotCharPos) : -1;
         if (hotKey != -1) {
-            hotKey = ASCII::toUpper(hotKey);
+            hotKey = toupper((unsigned char) hotKey);
 
             installAccelerator(hotKey, 0, this);
             if (xapp->AltMask != 0)
                 installAccelerator(hotKey, xapp->AltMask, this);
         }
     }
-    updateSize();
+    if (fIcon == null && fImage == null) {
+        updateSize();
+    }
 }
 
 void YButton::setPopup(YMenu *popup) {
@@ -460,7 +482,7 @@ void YButton::popdown() {
 }
 
 bool YButton::isFocusTraversable() {
-    return true;
+    return fEnabled;
 }
 
 void YButton::setAction(YAction action) {
@@ -472,7 +494,7 @@ void YButton::actionPerformed(YAction action, unsigned modifiers) {
         fListener->actionPerformed(action, modifiers);
 }
 
-ref<YFont> YButton::getFont() {
+YFont YButton::getFont() {
     return fPressed ? getActiveFont() : getNormalFont();
 }
 
