@@ -23,8 +23,11 @@
 IResourceLocator* YIcon::iconResourceLocator;
 
 YIcon::YIcon(upath filename) :
-        fSmall(null), fLarge(null), fHuge(null), loadedS(false), loadedL(false),
-        loadedH(false), fCached(false), fPath(filename.expand())
+    fPath(filename.expand()),
+    fSmall(null), fLarge(null), fHuge(null),
+    pSmall(None), pLarge(None), pHuge(None),
+    pOther(None), otherSize(0), fCached(false),
+    loadedS(false), loadedL(false), loadedH(false)
 {
     // don't attempt to load if icon is disabled
     if (fPath.equals("none") || fPath.equals("-"))
@@ -32,21 +35,39 @@ YIcon::YIcon(upath filename) :
 }
 
 YIcon::YIcon(ref<YImage> small, ref<YImage> large, ref<YImage> huge) :
-        fSmall(small), fLarge(large), fHuge(huge), loadedS(small != null),
-        loadedL(large != null), loadedH(huge != null), fCached(false),
-        fPath(null) {
+    fSmall(small), fLarge(large), fHuge(huge),
+    pSmall(None), pLarge(None), pHuge(None),
+    pOther(None), otherSize(0), fCached(false),
+    loadedS(small != null), loadedL(large != null), loadedH(huge != null)
+{
 }
 
 YIcon::~YIcon() {
+    if (xapp) {
+        if (pSmall)
+            XRenderFreePicture(xapp->display(), pSmall);
+        if (pLarge)
+            XRenderFreePicture(xapp->display(), pLarge);
+        if (pHuge)
+            XRenderFreePicture(xapp->display(), pHuge);
+        if (pOther)
+            XRenderFreePicture(xapp->display(), pOther);
+    }
 }
 
-static const char iconExts[][5] = {
-    ".png",
-    ".xpm",
-#ifdef ICE_SUPPORT_SVG
-    ".svg",
+bool YIcon::fSupportSVG;
+bool YIcon::fExamineSVG;
+bool YIcon::supportSVG() {
+    if (fExamineSVG == false) {
+        fExamineSVG = true;
+#if ICE_SUPPORT_SVG
+        fSupportSVG = true;
+#else
+        fSupportSVG = YImage::supportsFormat("svg");
 #endif
-};
+    }
+    return fSupportSVG;
+}
 
 static const char subcats[][12] = {
     "actions", "apps", "categories",
@@ -61,15 +82,12 @@ static const char subcats[][12] = {
     "legacy",
 };
 
-static bool hasImageExtension(const upath& base) {
-    auto baseExt(base.getExtension());
-    if (baseExt.length() != 4)
+static bool hasImageExtension(mstring base) {
+    if (base.length() <= 4)
         return false;
-    for (auto& pe : iconExts) {
-        if (baseExt.equals(pe, 4))
-            return true;
-    }
-    return false;
+    mstring ext(base.substring(base.length() - 4));
+    return ext == ".png" || ext == ".xpm" ||
+           (YIcon::supportSVG() && ext == ".svg");
 }
 
 class UniqueSizes {
@@ -83,12 +101,9 @@ public:
             // last resort
             + 128
             + 64
-            + 32
-
-#ifdef ICE_SUPPORT_SVG
-            + SCALABLE
-#endif
             ;
+        if (YIcon::supportSVG())
+            seen[last++] = SCALABLE;
     }
     bool have(unsigned size) {
         for (unsigned k : *this)
@@ -222,15 +237,15 @@ private:
         for (const char* entry : entries) {
             // try the scalable version if it can handle SVG
             if (strcmp(entry, "scalable") == 0) {
-#ifdef ICE_SUPPORT_SVG
-                auto& scaleCat = pools[fromResources].getCat(SCALABLE);
-                for (const char* sub : subcats) {
-                    mstring path(iconPathToken, "/scalable/", sub);
-                    if (upath(path).dirExists()) {
-                        ret += addPath(path, scaleCat);
+                if (YIcon::supportSVG()) {
+                    auto& scaleCat = pools[fromResources].getCat(SCALABLE);
+                    for (const char* sub : subcats) {
+                        mstring path(iconPathToken, "/scalable/", sub);
+                        if (upath(path).dirExists()) {
+                            ret += addPath(path, scaleCat);
+                        }
                     }
                 }
-#endif
             }
             else if (strcmp(entry, "base") == 0) {
                 mstring basePath(iconPathToken + "/base");
@@ -404,10 +419,12 @@ public:
                 mstring szSize(size);
                 basePath = mstring(basePath, "_", szSize, "x", szSize);
             }
-            for (auto& imgExt : iconExts) {
-                if (checkFile(basePath + imgExt))
-                    return true;
-            }
+            if (checkFile(basePath + ".png"))
+                return true;
+            if (checkFile(basePath + ".xpm"))
+                return true;
+            if (YIcon::supportSVG() && checkFile(basePath + ".svg"))
+                return true;
             return false;
         };
         auto checkFilesInFolder = [&](const mstring& dirPath, unsigned size,
@@ -503,16 +520,17 @@ ref<YImage> YIcon::loadIcon(unsigned size) {
             loadPath = findIcon(size);
         }
         if (loadPath != null) {
-            mstring cs(loadPath.path());
-            YTraceIcon trace(cs);
-
-#ifdef ICE_SUPPORT_SVG
-            mstring ext(loadPath.getExtension().lower());
-            if (ext == ".svg")
-                icon = YImage::loadsvg(cs);
-            else
+            YTraceIcon trace(loadPath.string());
+            if (loadPath.getExtension().lower() == ".svg") {
+#if ICE_SUPPORT_SVG
+                icon = YImage::loadsvg(loadPath);
+#else
+                if (supportSVG())
+                    icon = YImage::load(loadPath);
 #endif
-                icon = YImage::load(cs);
+            }
+            else
+                icon = YImage::load(loadPath);
         }
         else if (XDBG || YTrace::traces("icon")) {
             tlog("icon not found: %s %ux%u", fPath.string(), size, size);
@@ -635,33 +653,103 @@ void YIcon::freeIcons() {
     iconIndex = null;
 }
 
-unsigned YIcon::menuSize() {
-    return menuIconSize;
-}
-
-unsigned YIcon::smallSize() {
-    return smallIconSize;
-}
-
-unsigned YIcon::largeSize() {
-    return largeIconSize;
-}
-
-unsigned YIcon::hugeSize() {
-    return hugeIconSize;
-}
-
-bool YIcon::draw(Graphics& g, int x, int y, int size) {
-    ref<YImage> image = getScaledIcon(size);
-    if (image != null) {
-        if (!doubleBuffer) {
-            g.drawImage(image, x, y);
-        } else {
-            g.compositeImage(image, 0, 0, size, size, x, y);
+unsigned YIcon::fixIconSize(unsigned size) {
+    const int n = 8;
+    unsigned a[n] = { 16, 22, 24, 32, 48, 64, 128, 256 };
+    for (int i = 0; i < n; i++) {
+        if (a[i] == size)
+            return size;
+        if (a[i] > size) {
+            if (i > 0 && size - a[i - 1] <= a[i] - size)
+                return a[i - 1];
+            return a[i];
         }
+    }
+    return a[n - 1];
+}
+
+void YIcon::fixIconSizes() {
+    menuIconSize = fixIconSize(menuIconSize);
+    smallIconSize = fixIconSize(smallIconSize);
+    largeIconSize = fixIconSize(largeIconSize);
+    hugeIconSize = fixIconSize(hugeIconSize);
+}
+
+bool YIcon::draw(Graphics& g, int x, int y, unsigned size) {
+    ref<YImage> icon;
+    Picture pict = None;
+    if (size == smallSize()) {
+        if (pSmall == None) {
+            icon = small();
+            if (icon != null) {
+                ref<YPixmap> pixmap = icon->renderToPixmap(32, true);
+                if (pixmap != null) {
+                    pSmall = pixmap->picture();
+                    pixmap->forgetPicture();
+                }
+            }
+        }
+        pict = pSmall;
+    }
+    else if (size == largeSize()) {
+        if (pLarge == None) {
+            icon = large();
+            if (icon != null) {
+                ref<YPixmap> pixmap = icon->renderToPixmap(32, true);
+                if (pixmap != null) {
+                    pLarge = pixmap->picture();
+                    pixmap->forgetPicture();
+                }
+            }
+        }
+        pict = pLarge;
+    }
+    else if (size == hugeSize()) {
+        if (pHuge == None) {
+            icon = huge();
+            if (icon != null) {
+                ref<YPixmap> pixmap = icon->renderToPixmap(32, true);
+                if (pixmap != null) {
+                    pHuge = pixmap->picture();
+                    pixmap->forgetPicture();
+                }
+            }
+        }
+        pict = pHuge;
+    }
+    else if (size == otherSize && pOther) {
+        pict = pOther;
+    }
+    if (pict == None) {
+        icon = getScaledIcon(size);
+        if (icon != null) {
+            ref<YPixmap> pixmap = icon->renderToPixmap(32, true);
+            if (pixmap != null) {
+                pict = pixmap->picture();
+                pixmap->forgetPicture();
+                if (pOther) {
+                    XRenderFreePicture(xapp->display(), pOther);
+                }
+                pOther = pict;
+                otherSize = size;
+            }
+        }
+    }
+    if (pict) {
+        XRenderComposite(xapp->display(), PictOpOver,
+                         pict, None, g.picture(), 0, 0, 0, 0,
+                         x - g.xorigin(), y - g.yorigin(), size, size);
         return true;
     }
     return false;
 }
 
+void YIcon::setOther(Picture pict, unsigned size) {
+    if (pOther) {
+        XRenderFreePicture(xapp->display(), pOther);
+    }
+    pOther = pict;
+    otherSize = size;
+}
+
 // vim: set sw=4 ts=4 et:
