@@ -13,6 +13,7 @@
 #include "yicon.h"
 #include "wpixmaps.h"
 #include "yscrollview.h"
+#include "keysyms.h"
 
 #include "yxapp.h"
 #include "prefs.h"
@@ -20,7 +21,7 @@
 
 #include <string.h>
 
-static ref<YFont> listBoxFont;
+static YFont listBoxFont;
 static YColorName listBoxBg(&clrListBox);
 static YColorName listBoxFg(&clrListBoxText);
 static YColorName listBoxSelBg(&clrListBoxSelected);
@@ -30,12 +31,6 @@ inline int getIconSize() {
     return int(YIcon::smallSize());
 }
 
-inline ref<YFont> getListBoxFont() {
-    if (listBoxFont == null)
-        listBoxFont = YFont::getFont(XFA(listBoxFontName));
-    return listBoxFont;
-}
-
 int YListBox::fAutoScrollDelta = 0;
 
 int YListItem::getWidth() {
@@ -43,8 +38,8 @@ int YListItem::getWidth() {
     if (getIcon() != null) {
         width += getIconSize();
     }
-    if (getText() != null) {
-        width += getListBoxFont()->textWidth(getText()) + 3;
+    if (getText() != null && listBoxFont) {
+        width += listBoxFont->textWidth(getText()) + 3;
     }
     return width;
 }
@@ -69,7 +64,7 @@ YListBox::YListBox(YScrollView *view, YWindow *aParent):
     fGradient(null)
 {
     if (listBoxFont == null)
-        listBoxFont = YFont::getFont(XFA(listBoxFontName));
+        listBoxFont = listBoxFontName;
     addStyle(wsNoExpose);
     setBitGravity(NorthWestGravity);
     addEventMask(VisibilityChangeMask);
@@ -86,7 +81,6 @@ YListBox::YListBox(YScrollView *view, YWindow *aParent):
 
 YListBox::~YListBox() {
     fGradient = null;
-    listBoxFont = null;
 }
 
 bool YListBox::isFocusTraversable() {
@@ -159,12 +153,20 @@ int YListBox::findItemByPoint(int /*pX*/, int pY) {
     return (0 <= no && no < getItemCount()) ? no : -1;
 }
 
+int YListBox::findItem(const YListItem *item, int from, int stop) {
+    YListItem** ptr = &*fItems;
+    for (int i = from; i < stop; ++i)
+        if (ptr[i] == item)
+            return i;
+    return -1;
+}
+
 YListItem *YListBox::getItem(int no) {
     return (0 <= no && no < getItemCount()) ? fItems[no] : nullptr;
 }
 
 int YListBox::getLineHeight() {
-    return max(getIconSize(), listBoxFont->height()) + 2;
+    return max(getIconSize(), listBoxFont ? listBoxFont->height() : 0) + 2;
 }
 
 void YListBox::ensureVisibility(int item) { //!!! horiz too
@@ -245,19 +247,12 @@ void YListBox::repaint() {
 bool YListBox::handleKey(const XKeyEvent &key) {
     if (key.type == KeyPress) {
         KeySym k = keyCodeToKeySym(key.keycode);
+        k = mapKeypad(k);
         int m = KEY_MODMASK(key.state);
 
         bool clear = notbit(m, ControlMask);
         bool extend = hasbit(m, ShiftMask);
 
-        //int SelPos, OldPos = fFocusedItem, count = getItemCount();
-
-        //if (m & ShiftMask) {
-        //    SelPos = fFocusedItem;
-        //} else {
-        //    SelPos = -1;
-        //}
-
         switch (k) {
         case XK_Return:
         case XK_KP_Enter:
@@ -506,10 +501,10 @@ void YListBox::paintItem(Graphics &g, int n) {
     YListItem *a = getItem(n);
     int x = 3;
     int lh = getLineHeight();
-    int fh = getListBoxFont()->height();
+    int fh = listBoxFont ? listBoxFont->height() : 8;
     int xpos = 0;
     int y = n * lh;
-    int yPos = y + lh - (lh - fh) / 2 - listBoxFont->descent();
+    int yPos = y + lh - (lh - fh) / 2 - (listBoxFont ? listBoxFont->descent() : 0);
 
     if (a == nullptr)
         return ;
@@ -557,16 +552,13 @@ void YListBox::paintItem(Graphics &g, int n) {
 
     ref<YIcon> icon = a->getIcon();
     if (icon != null) {
-        ref<YImage> scaled = icon->getScaledIcon(getIconSize());
-        if (scaled != null) {
-            int dx = xpos + x - fOffsetX;
-            int dy = y - fOffsetY + 1;
-            g.drawImage(scaled, dx, dy);
-        }
+        int dx = xpos + x - fOffsetX;
+        int dy = y - fOffsetY + 1;
+        icon->draw(g, dx, dy, getIconSize());
     }
 
     mstring title(a->getText());
-    if (title != null) {
+    if (title != null && listBoxFont) {
         g.setColor(s ? listBoxSelFg : listBoxFg);
         g.setFont(listBoxFont);
         g.drawChars(title,
