@@ -18,7 +18,8 @@ bool YXApplication::popup(YWindow *forWindow, YPopupWindow *popup) {
 
         if (!grabEvents(forWindow ? forWindow : popup, cursor,
                         ButtonPressMask | ButtonReleaseMask |
-                        (menuMouseTracking ? PointerMotionMask : ButtonMotionMask)))
+                        ((menuMouseTracking || popup->pointerMotion())
+                        ? PointerMotionMask : ButtonMotionMask)))
         {
             return false;
         }
@@ -50,6 +51,7 @@ YPopupWindow::YPopupWindow(YWindow *aParent): YWindow(aParent) {
     fOwner = nullptr;
     fFlags = 0;
     fUp = false;
+    fPointerMotion = false;
     fXiScreen = -1;
     setStyle(wsSaveUnder | wsOverrideRedirect);
 }
@@ -184,6 +186,23 @@ bool YPopupWindow::popup(YWindow *owner,
         if (y < dy)
             y = dy;
     }
+    else {
+        YWindow* parent = forWindow->parent();
+        char* title = nullptr;
+        if (parent && parent->fetchTitle(&title)) {
+            if (0 == strcmp(title, "TitleBar")) {
+                int tx = 0;
+                int ty = parent->y();
+                parent->mapToGlobal(tx, ty);
+                int tw = int(parent->width());
+                if (x + int(width()) > tx + tw &&
+                    tx + int(width()) <= dx + dw &&
+                    tx + tw - int(width()) < dx + dw)
+                    x = max(dx, max(tx, tx + tw - int(width())));
+            }
+            XFree(title);
+        }
+    }
 
     setPosition(x, y);
 
