@@ -78,8 +78,8 @@ public:
     void setStyle(unsigned aStyle);
     void addStyle(unsigned aStyle) { setStyle(fStyle | aStyle); }
     unsigned getStyle() const { return fStyle; }
-    long getEventMask() const { return fEventMask; }
-    void addEventMask(long mask);
+    int getEventMask() const { return fEventMask; }
+    void addEventMask(int mask);
 
     void setVisible(bool enable);
     void show();
@@ -162,7 +162,7 @@ public:
     void setPointer(Cursor pointer);
     void grabKeyM(unsigned key, unsigned modifiers);
     void grabKey(unsigned key, unsigned modifiers);
-    unsigned VMod(unsigned modifiers);
+    static unsigned VMod(unsigned modifiers);
     void grabButtonM(int button, unsigned modifiers);
     void grabButton(int button, unsigned modifiers);
 
@@ -173,7 +173,6 @@ public:
 
     void paintExpose(int ex, int ey, int ew, int eh);
 
-    Graphics& getGraphics();
     virtual ref<YImage> getGradient() {
         return (parent() ? parent()->getGradient() : null); }
 
@@ -267,6 +266,7 @@ public:
     void clearWindow();
     void clearArea(int x, int y, unsigned w, unsigned h, bool expos = false);
     Pixmap createPixmap();
+    Pixmap createPixmap(unsigned width, unsigned height);
     XRenderPictFormat* format();
     Picture createPicture();
 
@@ -318,16 +318,17 @@ private:
     int fX, fY;
     unsigned fWidth, fHeight;
     int unmapCount;
-    Cursor fPointer;
-    Graphics *fGraphics;
-    long fEventMask;
+    int fEventMask;
     int fWinGravity, fBitGravity;
+    Cursor fPointer;
 
     struct YAccelerator {
         unsigned key;
         unsigned mod;
         YWindow *win;
         YAccelerator *next;
+        YAccelerator(unsigned k, unsigned m, YWindow* w, YAccelerator* n) :
+                     key(k), mod(m), win(w), next(n) { }
     };
 
     YAccelerator *accel;
